@@ -1,8 +1,15 @@
 import requests
 import json
 from base64 import b64encode
-import threading
+from collections import namedtuple
+from threading import Thread
 from time import time
+try:
+    import queue
+except ImportError:
+    import Queue as queue
+import select
+import socket
 
 baseAPIUrl = "https://api.crowdprocess.com/jobs/"
 
@@ -179,20 +186,89 @@ class Job(object):
                 yield n
                 self._batch_out[batch] += 1
 
-        t = threading.Thread(target=self.submit_tasks, args=(genwrapper(),))
+        results_req = requests.get(baseAPIUrl + self.id + "/results?stream",
+                                   stream=True,
+                                   headers=self._headers)
 
-        results = self.get_results_stream()
-        t.start()
+        errors_req = requests.get(baseAPIUrl + self.id + "/errors?stream",
+                                  stream=True,
+                                  headers=self._headers)
+
+        if results_req.status_code != requests.codes.ok:
+            results_req.raise_for_status()
+
+        if errors_req.status_code != requests.codes.ok:
+            errors_req.raise_for_status()
+
+        results_raw_req = results_req.raw
+        errors_raw_req = errors_req.raw
+        inputs = [results_raw_req, errors_raw_req]
+
+        results_queue = queue.Queue()
+        errors_queue = queue.Queue()
+
+        tasks = Thread(target=self.submit_tasks, args=(genwrapper(),))
+        tasks.daemon = True
+        tasks.start()
+
+        def get_results_and_errors():
+            while True:
+                inputready = []
+                try:
+                    inputready, _,_ = select.select(inputs, [], [], 0.001)
+                except select.error:
+                    break
+                except socket.error:
+                    break
+                except AttributeError:
+                    if results_raw_req.closed:
+                        if results_raw_req in inputs:
+                            inputs.remove(results_raw_req)
+                    if errors_raw_req.closed:
+                        if errors_raw_req in inputs:
+                            inputs.remove(errors_raw_req)
+                    if len(inputs) == 0:
+                        raise Exception("results and errors connections were closed unexpectedly")
+
+                for s in inputready:
+                    if s == results_raw_req:
+                        line = results_raw_req.readline()
+                        if len(line) == 0:
+                            continue
+                        results_queue.put(json.loads(line.decode()))
+                        self._batch_out[batch] -= 1
+
+                    if s == errors_raw_req:
+                        line = errors_raw_req.readline()
+                        if len(line) == 0:
+                            continue
+                        errors_queue.put(json.loads(line.decode()))
+                        self._batch_out[batch] -= 1
+
+                if not tasks.is_alive() and self._batch_out[batch] == 0:
+                    break
+
+
+        results_and_errors = Thread(target=get_results_and_errors)
+        results_and_errors.daemon = True
+        results_and_errors.start()
 
         def results_gen():
-            for result in results:
-                yield result
-                self._batch_out[batch] -= 1
-                while self._batch_out[batch] == 0 and t.isAlive():
-                    t.join(0.1)
+            while results_and_errors.is_alive() or not results_queue.empty():
+                try:
+                    yield results_queue.get(True, 0.001)
+                except queue.Empty:
+                    continue
+                results_queue.task_done()
 
-                if self._batch_out[batch] == 0 and not t.isAlive():
-                    break
-            t.join()
+        def errors_gen():
+            while results_and_errors.is_alive() or not errors_queue.empty():
+                try:
+                    yield errors_queue.get(True, 0.001)
+                except queue.Empty:
+                    continue
+                errors_queue.task_done()
 
-        return results_gen()
+        Data = namedtuple('Data', 'results, errors')
+
+        return Data(results_gen(), errors_gen())
